"""
开黑啦 v3 协议适配
============================

协议详情请看: `kaiheila V3`_

.. _kaiheila V3:
    https://developer.kaiheila.cn/doc/intro
"""

from .bot import Bot as Bot
from .event import Event as Event
from .adapter import Adapter as Adapter
from .message import Message as Message
from .message import MessageSegment as MessageSegment

